@@ -5,7 +5,6 @@
 //
 // SPDX-License-Identifier: GPL-2.0-or-later
 
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 import Atk from 'gi://Atk';
 import Clutter from 'gi://Clutter';
 import Gio from 'gi://Gio';
@@ -125,7 +124,10 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         else
             name = _('Favorites');
 
-        this.add_child(new St.Label({text: name}));
+        const label = new St.Label({text: name});
+        this.add_child(label);
+        this.actor.label_actor = label;
+
         this.connect('motion-event', this._onMotionEvent.bind(this));
         this.connect('notify::active', this._onActiveChanged.bind(this));
     }
@@ -214,9 +216,9 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._grab?.dismiss();
         delete this._grab;
 
-        let source = event.get_source();
-        if (source instanceof St.Widget)
-            source.sync_hover();
+        const targetActor = global.stage.get_event_actor(event);
+        if (targetActor instanceof St.Widget)
+            targetActor.sync_hover();
 
         return false;
     }
@@ -255,7 +257,7 @@ class DesktopTarget extends EventEmitter {
         this._desktopDestroyedId = 0;
 
         this._windowAddedId =
-            global.window_group.connect('actor-added',
+            global.window_group.connect('child-added',
                 this._onWindowAdded.bind(this));
 
         global.get_window_actors().forEach(a => {
@@ -363,6 +365,20 @@ class DesktopTarget extends EventEmitter {
     }
 }
 
+class MainLayout extends Clutter.BoxLayout {
+    static {
+        GObject.registerClass(this);
+    }
+
+    vfunc_get_preferred_height(container, forWidth) {
+        const [mainChild] = container;
+        const [minHeight, natHeight] =
+            mainChild.get_preferred_height(forWidth);
+
+        return [minHeight, natHeight + MENU_HEIGHT_OFFSET];
+    }
+}
+
 class ApplicationsButton extends PanelMenu.Button {
     static {
         GObject.registerClass(this);
@@ -380,12 +396,12 @@ class ApplicationsButton extends PanelMenu.Button {
         this.accessible_role = Atk.Role.LABEL;
 
         this._label = new St.Label({
-            text: _('Applications'),
+            text: _('Apps'),
             y_expand: true,
             y_align: Clutter.ActorAlign.CENTER,
         });
 
-        this.add_actor(this._label);
+        this.add_child(this._label);
         this.name = 'panelApplications';
         this.label_actor = this._label;
 
@@ -432,15 +448,6 @@ class ApplicationsButton extends PanelMenu.Button {
         }
     }
 
-    _createVertSeparator() {
-        let separator = new St.DrawingArea({
-            style_class: 'calendar-vertical-separator',
-            pseudo_class: 'highlighted',
-        });
-        separator.connect('repaint', this._onVertSepRepaint.bind(this));
-        return separator;
-    }
-
     _onDestroy() {
         super._onDestroy();
 
@@ -462,21 +469,6 @@ class ApplicationsButton extends PanelMenu.Button {
         return super._onMenuKeyPress(actor, event);
     }
 
-    _onVertSepRepaint(area) {
-        let cr = area.get_context();
-        let themeNode = area.get_theme_node();
-        let [width, height] = area.get_surface_size();
-        let stippleColor = themeNode.get_color('-stipple-color');
-        let stippleWidth = themeNode.get_length('-stipple-width');
-        let x = Math.floor(width / 2) + 0.5;
-        cr.moveTo(x, 0);
-        cr.lineTo(x, height);
-        Clutter.cairo_set_source_color(cr, stippleColor);
-        cr.setDash([1, 3], 1); // Hard-code for now
-        cr.setLineWidth(stippleWidth);
-        cr.stroke();
-    }
-
     _onOpenStateChanged(menu, open) {
         if (open) {
             if (this.reloadFlag) {
@@ -503,7 +495,7 @@ class ApplicationsButton extends PanelMenu.Button {
                 let id;
                 try {
                     id = entry.get_desktop_file_id(); // catch non-UTF8 filenames
-                } catch (e) {
+                } catch {
                     continue;
                 }
                 let app = appSys.lookup_app(id);
@@ -522,7 +514,7 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 
     scrollToButton(button) {
-        let appsScrollBoxAdj = this.applicationsScrollBox.get_vscroll_bar().get_adjustment();
+        let appsScrollBoxAdj = this.applicationsScrollBox.get_vadjustment();
         let appsScrollBoxAlloc = this.applicationsScrollBox.get_allocation_box();
         let currentScrollValue = appsScrollBoxAdj.get_value();
         let boxHeight = appsScrollBoxAlloc.y2 - appsScrollBoxAlloc.y1;
@@ -537,7 +529,7 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 
     scrollToCatButton(button) {
-        let catsScrollBoxAdj = this.categoriesScrollBox.get_vscroll_bar().get_adjustment();
+        let catsScrollBoxAdj = this.categoriesScrollBox.get_vadjustment();
         let catsScrollBoxAlloc = this.categoriesScrollBox.get_allocation_box();
         let currentScrollValue = catsScrollBoxAdj.get_value();
         let boxHeight = catsScrollBoxAlloc.y2 - catsScrollBoxAlloc.y1;
@@ -554,43 +546,29 @@ class ApplicationsButton extends PanelMenu.Button {
     _createLayout() {
         let section = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(section);
-        this.mainBox = new St.BoxLayout({vertical: false});
+        this.mainBox = new St.BoxLayout({layoutManager: new MainLayout()});
         this.leftBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox = new St.ScrollView({
             style_class: 'apps-menu vfade',
             x_expand: true,
         });
-        this.applicationsScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
-        let vscroll = this.applicationsScrollBox.get_vscroll_bar();
-        vscroll.connect('scroll-start', () => {
-            this.menu.passEvents = true;
-        });
-        vscroll.connect('scroll-stop', () => {
-            this.menu.passEvents = false;
-        });
         this.categoriesScrollBox = new St.ScrollView({
             style_class: 'vfade',
         });
-        this.categoriesScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
-        vscroll = this.categoriesScrollBox.get_vscroll_bar();
-        vscroll.connect('scroll-start', () => (this.menu.passEvents = true));
-        vscroll.connect('scroll-stop', () => (this.menu.passEvents = false));
         this.leftBox.add_child(this.categoriesScrollBox);
 
         this.applicationsBox = new St.BoxLayout({vertical: true});
-        this.applicationsScrollBox.add_actor(this.applicationsBox);
+        this.applicationsScrollBox.set_child(this.applicationsBox);
         this.categoriesBox = new St.BoxLayout({vertical: true});
-        this.categoriesScrollBox.add_actor(this.categoriesBox);
+        this.categoriesScrollBox.set_child(this.categoriesBox);
 
-        this.mainBox.add(this.leftBox);
-        this.mainBox.add_child(this._createVertSeparator());
+        this.mainBox.add_child(this.leftBox);
         this.mainBox.add_child(this.applicationsScrollBox);
-        section.actor.add_actor(this.mainBox);
+        section.actor.add_child(this.mainBox);
     }
 
     _display() {
         this._applicationsButtons.clear();
-        this.mainBox.style = 'width: 35em;';
         this.mainBox.hide();
 
         // Load categories
@@ -598,7 +576,7 @@ class ApplicationsButton extends PanelMenu.Button {
         this._tree.load_sync();
         let root = this._tree.get_root_directory();
         let categoryMenuItem = new CategoryMenuItem(this, null);
-        this.categoriesBox.add_actor(categoryMenuItem);
+        this.categoriesBox.add_child(categoryMenuItem);
         let iter = root.iter();
         let nextType;
         while ((nextType = iter.next()) !== GMenu.TreeItemType.INVALID) {
@@ -614,18 +592,12 @@ class ApplicationsButton extends PanelMenu.Button {
             this._loadCategory(categoryId, dir);
             if (this.applicationsByCategory[categoryId].length > 0) {
                 categoryMenuItem = new CategoryMenuItem(this, dir);
-                this.categoriesBox.add_actor(categoryMenuItem);
+                this.categoriesBox.add_child(categoryMenuItem);
             }
         }
 
         // Load applications
         this._displayButtons(this._listApplications(null));
-
-        let themeContext = St.ThemeContext.get_for_stage(global.stage);
-        let scaleFactor = themeContext.scale_factor;
-        let categoriesHeight = this.categoriesBox.height / scaleFactor;
-        let height = Math.round(categoriesHeight) + MENU_HEIGHT_OFFSET;
-        this.mainBox.style += `height: ${height}px`;
     }
 
     selectCategory(dir) {
@@ -633,7 +605,7 @@ class ApplicationsButton extends PanelMenu.Button {
             if (c._delegate instanceof PopupMenu.PopupSeparatorMenuItem)
                 c._delegate.destroy();
             else
-                this.applicationsBox.remove_actor(c);
+                this.applicationsBox.remove_child(c);
         });
 
         if (dir)
@@ -656,7 +628,7 @@ class ApplicationsButton extends PanelMenu.Button {
                 this._applicationsButtons.set(app, item);
             }
             if (!item.get_parent())
-                this.applicationsBox.add_actor(item);
+                this.applicationsBox.add_child(item);
         }
     }
 
