@@ -226,12 +226,10 @@ class BaseButton extends DashItemContainer {
         return this._button.has_style_class_name('focused');
     }
 
-    // eslint-disable-next-line camelcase
     get ignore_workspace() {
         return this._ignoreWorkspace;
     }
 
-    // eslint-disable-next-line camelcase
     set ignore_workspace(ignore) {
         if (this._ignoreWorkspace === ignore)
             return;
@@ -772,11 +770,20 @@ class WindowList extends St.Widget {
             () => this._onWorkspaceMenuSet(), this);
         this._onWorkspaceMenuSet();
 
-        const chromeOptions = {
+        const inOverview = Main.overview.visible ||
+            (Main.layoutManager._startingUp && Main.sessionMode.hasOverview);
+
+        const overviewChromeOptions = {
             affectsStruts: true,
+        };
+        const chromeOptions = {
+            ...overviewChromeOptions,
             trackFullscreen: true,
         };
-        Main.layoutManager.addChrome(this, chromeOptions);
+        Main.layoutManager.addChrome(this, inOverview
+            ? overviewChromeOptions
+            : chromeOptions);
+
         Main.uiGroup.set_child_above_sibling(this, Main.layoutManager.panelBox);
         Main.ctrlAltTabManager.addGroup(this, _('Window List'), 'start-here-symbolic');
 
@@ -813,20 +820,17 @@ class WindowList extends St.Widget {
 
         Main.overview.connectObject(
             'showing', () => {
-                Main.layoutManager.untrackChrome(this);
+                this._retrackChrome(overviewChromeOptions);
                 this.hide();
                 this._updateKeyboardAnchor();
             },
             'hidden', () => {
-                Main.layoutManager.trackChrome(this, chromeOptions);
+                this._retrackChrome(chromeOptions);
                 this.visible = !this._monitor.inFullscreen;
                 this._updateKeyboardAnchor();
             }, this);
 
         global.display.connectObject('in-fullscreen-changed', () => {
-            // Work-around for initial change from unknown to !fullscreen
-            if (Main.overview.visible)
-                this.hide();
             this._updateKeyboardAnchor();
         }, this);
 
@@ -881,6 +885,11 @@ class WindowList extends St.Widget {
             this._monitor.y + this._monitor.height - this.height);
     }
 
+    _retrackChrome(options) {
+        Main.layoutManager.untrackChrome(this);
+        Main.layoutManager.trackChrome(this, options);
+    }
+
     _updateWorkspaceIndicatorVisibility() {
         let workspaceManager = global.workspace_manager;
         let hasWorkspaces = this._mutterSettings.get_boolean('dynamic-workspaces') ||
